# -*- coding: utf-8 -*-
# Copyright © 2021 Wacom. All rights reserved.
import abc
import enum
from typing import List, Dict, Any, NewType, Optional

#  ---------------------------------------- Type definitions -----------------------------------------------------------
LanguageCode = NewType("LanguageCode", str)
ReferenceId = NewType("ReferenceId", str)


#  ---------------------------------------- Exceptions -----------------------------------------------------------------
class ServiceException(Exception):
    """Service exception."""
    pass


class KnowledgeException(Exception):
    """Knowledge exception."""
    pass


#  ---------------------------------------- Constants ------------------------------------------------------------------
VND_WACOM_INK_MODEL: str = 'application/vnd.wacom-knowledge.model'
RDF_SYNTAX_NS_TYPE: str = 'http://www.w3.org/1999/02/22-rdf-syntax-ns#type'
RDF_SCHEMA_COMMENT: str = 'http://www.w3.org/2000/01/rdf-schema#comment'
RDF_SCHEMA_LABEL: str = 'http://www.w3.org/2000/01/rdf-schema#label'
ALIAS_TAG: str = 'alias'
DATA_PROPERTY_TAG: str = 'literal'
VALUE_TAG: str = 'value'
LANGUAGE_CODE_TAG: str = 'languageCode'
LOCALE_TAG: str = 'locale'
DATA_PROPERTIES_TAG: str = 'literals'
SOURCE_REFERENCE_ID_TAG: str = 'source_reference_id'
SOURCE_SYSTEM_TAG: str = 'source_system'
OBJECT_PROPERTIES_TAG: str = 'relations'
OWNER_TAG: str = 'owner'
LOCALIZED_CONTENT_TAG: str = 'LocalizedContent'
STATUS_FLAG_TAG: str = 'status'
CONTENT_TAG: str = 'value'
URI_TAG: str = 'uri'
TEXT_TAG: str = 'text'
TYPE_TAG: str = 'type'
IMAGE_TAG: str = 'image'
DESCRIPTION_TAG: str = 'description'
COMMENT_TAG: str = 'text'
DESCRIPTIONS_TAG: str = 'descriptions'
RELATIONS_TAG: str = 'relations'
LABELS_TAG: str = 'labels'
IS_MAIN_TAG: str = 'isMain'
DATA_TYPE_TAG: str = 'dataType'
RELATION_TAG: str = 'relation'
OUTGOING_TAG: str = 'out'
INCOMING_TAG: str = 'in'
TENANT_RIGHTS_TAG: str = 'tenantRights'

# Mapping to map the simple language_code code to a default language_code / country code
language_code_mapping: Dict[str, LanguageCode] = {
    'en': LanguageCode('en_US'),
    'ja': LanguageCode('ja_JP'),
    'de': LanguageCode('de_DE'),
    'bg': LanguageCode('bg_BG')
}


class EntityStatus(enum.Enum):
    """
    Entity Status
    -------------
    Status of the entity synchronization (client and knowledge graph).
    """
    UNKNOWN = 0
    """Unkown status."""
    CREATED = 1
    """Entity has been created and not yet update."""
    UPDATED = 2
    """Entity has been updated by the client and must be synced."""
    SYNCED = 3
    """State of entity is in sync with knowledge graph."""


class LocalizedContent(abc.ABC):
    """
    Localized content
    -----------------
    Content that is multi-lingual.

    Parameters
    ----------
    content: str
        Content value
    language_code: LanguageCode (default:= 'en_US')
        ISO-3166 Country Codes and ISO-639 Language Codes in the format '<language_code>_<country>, e.g., en_US.
    """

    def __init__(self, content: str, language_code: LanguageCode = 'en_US'):
        self.__content: str = content
        self.__language_code: LanguageCode = language_code

    @property
    def content(self) -> str:
        """String representation of the content."""
        return self.__content

    @content.setter
    def content(self, value: str):
        self.__content = value

    @property
    def language_code(self) -> LanguageCode:
        """Language code of the content."""
        return self.__language_code

    def __repr__(self):
        return f'{self.content}@{self.language_code}'


class Label(LocalizedContent):
    """
    Label
    -----
    Label that is multi-lingual.

    Parameters
    ----------
    content: str
        Content value
    language_code: LanguageCode (default:= 'en_US')
        Language code of content
    main: bool (default:=False)
        Main content
    """
    def __init__(self, content: str, language_code: LanguageCode = 'en_US', main: bool = False):
        self.__main: bool = main
        super().__init__(content, language_code)

    @property
    def main(self) -> bool:
        """Flag if the content is the  main content or an alias."""
        return self.__main

    @staticmethod
    def create_from_dict(dict_label: Dict[str, Any], tag_name: str = CONTENT_TAG, locale_name: str = LANGUAGE_CODE_TAG)\
            -> 'Label':
        if tag_name not in dict_label:
            raise ValueError("Dict is does not contain a localized label.")
        if locale_name not in dict_label:
            raise ValueError("Dict is does not contain a language code")
        if IS_MAIN_TAG in dict_label:
            return Label(dict_label[tag_name], dict_label[locale_name], dict_label[IS_MAIN_TAG])
        else:
            return Label(dict_label[tag_name], dict_label[locale_name])

    @staticmethod
    def create_from_list(param: List[dict]) -> List[LOCALIZED_CONTENT_TAG]:
        return [Label.create_from_dict(p) for p in param]

    def __dict__(self):
        return {
            CONTENT_TAG: self.content,
            LANGUAGE_CODE_TAG: self.language_code,
            IS_MAIN_TAG: self.main
        }


class Description(LocalizedContent):
    """
    Description
    -----------
    Description that is multi-lingual.

    Parameters
    ----------
    description: str
        Description value
    language_code: LanguageCode (default:= 'en_US')
        Language code of content
    """

    def __init__(self, description: str, language_code: LanguageCode = 'en_US'):
        super().__init__(description, language_code)

    @staticmethod
    def create_from_dict(dict_description: Dict[str, Any]) -> 'Description':
        if DESCRIPTION_TAG not in dict_description or LANGUAGE_CODE_TAG not in dict_description:
            raise ValueError("Dict is does not contain a localized label.")
        return Description(dict_description[DESCRIPTION_TAG], dict_description[LANGUAGE_CODE_TAG])

    @staticmethod
    def create_from_list(param: List[Dict[str, Any]]) -> List['Description']:
        return [Description.create_from_dict(p) for p in param]

    def __dict__(self):
        return {
            DESCRIPTION_TAG: self.content,
            LANGUAGE_CODE_TAG: self.language_code,
        }


class Comment(LocalizedContent):
    """
    Comment
    -------
    Comment that is multi-lingual.

    Parameters
    ----------
    text: str
        Text value
    language_code: LanguageCode (default:= 'en_US')
        Language code of content
    """

    def __init__(self, text: str, language_code: LanguageCode = 'en_US'):
        super().__init__(text, language_code)

    @staticmethod
    def create_from_dict(dict_description: Dict[str, Any]) -> 'Comment':
        if DESCRIPTION_TAG not in dict_description or LANGUAGE_CODE_TAG not in dict_description:
            raise ValueError("Dict is does not contain a localized comment.")
        return Comment(dict_description[DESCRIPTION_TAG], dict_description[LANGUAGE_CODE_TAG])

    @staticmethod
    def create_from_list(param: List[Dict[str, Any]]) -> List['Description']:
        return [Description.create_from_dict(p) for p in param]

    def __dict__(self):
        return {
            DESCRIPTION_TAG: self.content,
            LANGUAGE_CODE_TAG: self.language_code,
        }


class OntologyObject(abc.ABC):
    """
    Generic ontology object
    -----------------------

    Parameters
    ----------
    tenant_id: str
        Reference id for tenant
    iri: str
        IRI of the ontology object
    icon: str
        Icon assigned to object, visually representing it
    labels: List[Label]
        List of multi-language_code labels
    comments: List[Label]
        List of multi-language_code comments
    context: str
        Context
    """

    def __init__(self, tenant_id: str, iri: str, icon: str, labels: List[Label],
                 comments: List[Comment], context: str):
        self.__tenant_id: str = tenant_id
        self.__labels: List[Label] = labels
        self.__comments: List[Comment] = comments
        self.__iri: str = iri
        self.__icon: str = icon
        self.__context: str = context

    @property
    def tenant_id(self) -> str:
        """Tenant id."""
        return self.__tenant_id

    @property
    def iri(self) -> str:
        """IRI """
        return self.__iri

    @property
    def context(self) -> str:
        """Context."""
        return self.__context

    @property
    def icon(self) -> str:
        """Icon."""
        return self.__icon

    @icon.setter
    def icon(self, value: str):
        self.__icon = value

    @property
    def labels(self) -> List[Label]:
        return self.__labels

    def label_for_lang(self, language_code: LanguageCode) -> Optional[Label]:
        for label in self.labels:
            if label.language_code == language_code:
                return label
        return None

    @property
    def comments(self) -> List[Comment]:
        return self.__comments

    def comment_for_lang(self, language_code: LanguageCode) -> Optional[Comment]:
        for comment in self.comments:
            if comment.language_code == language_code:
                return comment
        return None


class OntologyContextSettings(object):
    """
    OntologyContextSettings
    -----------------------
    Describes the settings of the context, such as:
    - prefixes for RDF, RDFS and OWL
    - Base literal URI
    - Base class URI
    - Description literal name
    - depth
    """

    def __init__(self, rdf_prefix: str, rdfs_prefix: str, owl_prefix: str, base_literal_uri: str, base_class_uri: str,
                 description_literal_name: str, depth: int):
        self.__rdf_prefix: str = rdf_prefix
        self.__rdfs_prefix: str = rdfs_prefix
        self.__owl_prefix: str = owl_prefix
        self.__base_literal_uri: str = base_literal_uri
        self.__base_class_uri: str = base_class_uri
        self.__description_literal_name: str = description_literal_name
        self.__depth: int = depth

    @property
    def rdf_prefix(self):
        """RDF prefix"""
        return self.__rdf_prefix

    @property
    def rdfs_prefix(self):
        """RDFS prefix"""
        return self.__rdfs_prefix

    @property
    def owl_prefix(self):
        """OWL prefix"""
        return self.__owl_prefix

    @property
    def base_literal_uri(self):
        """Base literal URI."""
        return self.__base_literal_uri

    @property
    def base_class_uri(self):
        """Base class URI."""
        return self.__base_class_uri

    @property
    def description_literal_name(self) -> str:
        """Literal name of the description."""
        return self.__description_literal_name

    @property
    def depth(self) -> int:
        """Depth."""
        return self.__depth


class OntologyContext(OntologyObject):
    """
    OntologyContext
    ----------------
    Ontology context representation.

    Parameters
    ----------
    cid: str
        Context id
    tenant_id: str
        Tenant id.
    name: str
        Name of the ontology context
    icon: str
        Icon or Base64 encoded
    labels: List[Label]
        List of labels
    comments: List[Comment]
        List of comments
    last_update:
        Last update
    context: str
        context name
    base_uri: str
        Base URI
    """

    def __init__(self, cid: str, tenant_id: str, name: str, icon: str, labels: List[Label],
                 comments: List[Comment], last_update: str, context: str, base_uri: str, version: int,
                 settings: OntologyContextSettings):
        self.__id = cid
        self.__base_uri: str = base_uri
        self.__version: int = version
        self.__last_update: str = last_update
        self.__settings: OntologyContextSettings = settings
        super().__init__(tenant_id, name, icon, labels, comments, context)

    @property
    def id(self) -> str:
        return self.__id

    @property
    def base_uri(self) -> str:
        return self.__base_uri

    @classmethod
    def from_dict(cls, context_dict: Dict[str, Any]):
        context_data: Dict[str, Any] = context_dict['data']
        context_settings: Dict[str, Any] = context_data['settings']
        labels: List[Label] = [] if context_data['labels'] is None else \
            [Label(content=la['text'], language_code=la['locale']) for la in context_data['labels']]
        comments: List[Comment] = [] if context_data['comments'] is None else \
            [Comment(text=la['text'], language_code=la['locale']) for la in context_data['comments']]
        settings: OntologyContextSettings = OntologyContextSettings(
            context_settings['rdfPrefix'], context_settings['rdfsPrefix'], context_settings['owlPrefix'],
            context_settings['baseLiteralUri'], context_settings['baseClassUri'],
            context_settings['descriptionLiteralName'], context_settings['depth']
        )
        return OntologyContext(context_data['id'], context_data['tenantId'], context_data['name'],
                               context_data['icon'], labels, comments, context_data['lastUpdated'],
                               context_data['context'], context_data['baseURI'], context_dict['version'],
                               settings)

    def __repr__(self):
        return f'<OntologyContext> - [id:={self.id}, iri:={self.iri}]'



# -*- coding: utf-8 -*-
# Copyright © 2021 Wacom. All rights reserved.
from abc import ABC

import requests
from requests import Response


class WacomServiceException(Exception):
    """Exception thrown if Wacom service fails."""


class RESTAPIClient(ABC):
    """
    Abstract REST API client
    ------------------------
    REST API client handling the service url.

    Arguments
    ---------
    service_url: str
        Service URL for service
    verify_calls: bool (default:= False)
        Flag if the service calls should be verified
    """
    def __init__(self, service_url: str, verify_calls: bool = False):
        self.__service_url: str = service_url
        self.__verify_calls: bool = verify_calls

    @property
    def service_url(self) -> str:
        """Service URL."""
        return self.__service_url

    @property
    def verify_calls(self):
        """Certificate verification activated."""
        return self.__verify_calls

    @verify_calls.setter
    def verify_calls(self, value: bool):
        self.__verify_calls = value


class WacomServiceAPIClient(RESTAPIClient):
    """
    Wacom Service API Client
    ------------------------
    Abstract class for Wacom service APIs.

    Parameters
    ----------
    application_name: str
        Name of the application using the service
    service_url: str
        URL of the service
    service_endpoint: str
        Base endpoint
    verify_calls: bool (Default:= False)
        Flag if  API calls should be verified.
    """
    AUTH_ENDPOINT: str = 'auth/user'
    USER_ENDPOINT: str = 'user'
    USER_LOGIN_ENDPOINT: str = f'{USER_ENDPOINT}/login'

    def __init__(self, application_name: str, service_url: str, service_endpoint: str, verify_calls: bool = False):
        self.__application_name: str = application_name
        self.__service_endpoint: str = service_endpoint
        super().__init__(service_url, verify_calls)

    def request_user_token(self, tenant_key: str, external_id: str) -> str:
        """
        Login as user by using the tenant key and its external user id.

        Parameters
        ----------
        tenant_key: str
            Tenant key
        external_id: str
            External id.

        Returns
        -------
        auth_key: str
            Authentication key for identifying the user for the service calls.

        Raises
        ------
        WacomServiceException
            Exception if service returns HTTP error code.
        """
        url: str = f'{self.service_url}/graphdata/{WacomServiceAPIClient.USER_LOGIN_ENDPOINT}/'
        headers: dict = {
            'x-tenant-api-key': tenant_key
        }
        payload: dict = {
            'externalUserId': external_id
        }
        response: Response = requests.post(url, headers=headers, json=payload, verify=self.verify_calls)
        if response.ok:
            return response.text
        raise WacomServiceException(f'Response code:={response.status_code}, exception:= {response.text}')

    @property
    def service_endpoint(self):
        """Service endpoint."""
        return '' if len(self.__service_endpoint) == 0 else f'{self.__service_endpoint}/'

    @property
    def service_base_url(self):
        """Service endpoint."""
        return f'{self.service_url}/{self.service_endpoint}'

    @property
    def application_name(self):
        """Application name."""
        return self.__application_name



# -*- coding: utf-8 -*-
# Copyright © 2021 Wacom. All rights reserved.
import abc
from enum import Enum
from typing import List, Optional

from knowledge.base.entity import LanguageCode
from knowledge.base.ontology import THING_CLASS, OntologyClassReference
from knowledge.services.base import WacomServiceAPIClient, RESTAPIClient


class EntityType(Enum):
    """
    Entity types
    ------------
    Different types of entities.
    """
    PUBLIC_ENTITY = 0
    """Public entity - Entity from a public knowledge graph"""
    PERSONAL_ENTITY = 1
    """Personal entity - Entity from a personal / organisational knowledge graph"""
    NAMED_ENTITY = 2
    """Simple entity - Entity type not linked to a knowledge graph"""


class KnowledgeSource(Enum):
    """
    Knowledge source
    ----------------
    List of knowledge sources which a used within Semantic Ink.
    """
    WIKIDATA = 'wikidata'
    """Wikidata"""
    DBPEDIA = 'dbpedia'
    """dbpedia"""
    WACOM_KNOWLEDGE = 'wacom'
    """Wacom Personal Knowledge"""


class BasicType(Enum):
    """
    Basic type
    ----------
    Basic type of entities.
    """
    UNKNOWN = 'Unknown'
    MONEY = 'Money'
    PERSON = 'Person'
    DATE = 'Date'
    PLACE = 'Place'
    TIME = 'Time'
    NUMBER = 'Number'


class EntitySource(object):
    """
    EntitySource
    ------------
    Source of the entity.

    Parameters
    ----------
    uri: str
        URI of entity
    source: KnowledgeSource
        Identifier where the entity originates.
    """

    def __init__(self, uri: str, source: KnowledgeSource):
        self.__uri = uri
        self.__source: KnowledgeSource = source

    @property
    def uri(self) -> str:
        """Identifier with the knowledge graph."""
        return self.__uri

    @property
    def source(self) -> KnowledgeSource:
        """Source of the entity."""
        return self.__source

    def __repr__(self):
        return f'{self.uri} ({self.source})'


class NamedEntity(abc.ABC):
    """
    NamedEntity
    -----------
    A named entity which is recognized by recognition engine.
    The class contains information on the found entity, found in reference text.

    Parameters
    ----------
    ref_text: str
        Reference text. Entity found for this specific text
    start_idx: int
        Start index within the full reference text
    end_idx: int
        End index with the full reference text
    entity_type: EntityType
        Type of the entity.
    """

    def __init__(self, ref_text: str, start_idx: int, end_idx: int, entity_type: EntityType):
        self.__ref_text: str = ref_text
        self.__start_idx: int = start_idx
        self.__end_idx: int = end_idx
        self.__type: EntityType = entity_type

    @property
    def ref_text(self) -> str:
        """Reference text for which the entity has been found"""
        return self.__ref_text

    @property
    def start_idx(self) -> int:
        """Start index within the text handed to the named entity recognition."""
        return self.__start_idx

    @property
    def end_idx(self) -> int:
        """End index within the text handed to the named entity recognition."""
        return self.__end_idx

    @property
    def entity_type(self) -> EntityType:
        """Type of the entity."""
        return self.__type

    def __repr__(self):
        return f'{self.ref_text} [{self.start_idx}-{self.end_idx}'


class KnowledgeGraphEntity(NamedEntity):
    """
    Knowledge graph entity
    ----------------------
    Entity from a knowledge graph.

    Parameters
    ----------
    ref_text: str
        Reference text. Entity found for this specific text
    start_idx: int
        Start index within the full reference text
    end_idx: int
        End index with the full reference text
    label: str
        Main label of the entity.
    confidence: float
        Confidence value if available
    source: EntitySource
        Source of the entity
    content_link: str
        Link to side with content
    ontology_types: List[str]
        List of ontology types (class names)
    entity_type: EntityType
        Type of the entity.
    """

    def __init__(self, ref_text: str, start_idx: int, end_idx: int, label: str, confidence: float,
                 source: EntitySource, content_link: str,
                 ontology_types: List[str], entity_type: EntityType = EntityType.PUBLIC_ENTITY):
        super().__init__(ref_text, start_idx, end_idx, entity_type)
        self.__source: EntitySource = source
        self.__content_link: str = content_link
        self.__label: str = label
        self.__confidence: float = confidence
        self.__description: Optional[str] = None
        self.__thumbnail: Optional[str] = None
        self.__ontology_types: List[str] = ontology_types
        self.__relevant_type: OntologyClassReference = THING_CLASS

    @property
    def entity_source(self) -> EntitySource:
        """Source of the entity."""
        return self.__source

    @property
    def label(self) -> str:
        """Label of the entity from the knowledge graph."""
        return self.__label

    @property
    def confidence(self) -> float:
        """Confidence level of the system that links the entities."""
        return self.__confidence

    @confidence.setter
    def confidence(self, value: float):
        self.__confidence = value

    @property
    def description(self) -> Optional[str]:
        """Description of the entity if available."""
        return self.__description

    @description.setter
    def description(self, value: str):
        self.__description = value

    @property
    def thumbnail(self) -> Optional[str]:
        """Thumbnail to describes the entity."""
        return self.__thumbnail

    @thumbnail.setter
    def thumbnail(self, value: str):
        self.__thumbnail = value

    @property
    def content_link(self) -> str:
        """Link to content page."""
        return self.__content_link

    @property
    def ontology_types(self) -> List[str]:
        """List of ontology types."""
        return self.__ontology_types

    @property
    def relevant_type(self) -> OntologyClassReference:
        """"Most relevant ontology type. That likes to Wacom's personal knowledge base ontology."""
        return self.__relevant_type

    @relevant_type.setter
    def relevant_type(self, value: OntologyClassReference):
        self.__relevant_type = value

    def __repr__(self):
        return f'{self.ref_text} [{self.start_idx}-{self.end_idx}] -> {self.entity_source} [{self.entity_type}]'


class BasicNamedEntity(NamedEntity):
    """
    Basic named entity
    ------------------
    Entity found by Named entity recognition.

    Parameters
    ----------
    ref_text: str
        Reference text. Entity found for this specific text
    start_idx: int
        Start index within the full reference text
    end_idx: int
        End index with the full reference text
    basic_type: BasicType
        Type of the entity.
    """

    def __init__(self, ref_text: str, start_idx: int, end_idx: int, basic_type: BasicType):
        super().__init__(ref_text, start_idx, end_idx, EntityType.NAMED_ENTITY)
        self.__basic_type: BasicType = basic_type

    @property
    def basic_type(self) -> BasicType:
        """Basic type that is recognized."""
        return self.__basic_type

    def __repr__(self):
        return f'{self.ref_text} [{self.start_idx}-{self.end_idx}] -> {self.basic_type}'


class PersonalEntityLinkingProcessor(WacomServiceAPIClient):
    """
    PersonalEntityLinkingProcessor
    ------------------------------
    Service that links entities to a entities in a personal knowledge graph.

    Parameters
    ----------
    service_url: str
        URL where the service has been deployed
    supported_languages: List[str] = None
        List of supported languages
    verify_calls: bool (default:=False)
        Verifies all HTTPS calls and the associated certificate.
    """

    def __init__(self, service_url: str = str, supported_languages: List[str] = None, verify_calls: bool = False):
        super().__init__(application_name="Personal entity linking", service_url=service_url,
                         service_endpoint="graphdata", verify_calls=verify_calls)
        self.__supported_languages: List[str] = supported_languages if supported_languages else []

    @abc.abstractmethod
    def link_personal_entities(self, auth_key: str, text: str, language_code: LanguageCode = 'en_US') \
            -> List[KnowledgeGraphEntity]:
        """
        Performs Named Entity Linking on a text. It only finds entities which are accessible by the user identified by
        the auth key.

        Parameters
        ----------
        auth_key: str
            Auth key identifying a user within the Wacom personal knowledge service.
        text: str
            Text where the entities shall be tagged in.
        language_code: LanguageCode
            ISO-3166 Country Codes and ISO-639 Language Codes in the format '<language_code>_<country>, e.g., en_US.

        Returns
        -------
        entities: List[KnowledgeGraphEntity]
            List of knowledge graph entities.
        """
        raise NotImplemented

    @property
    def supported_language(self) -> List[str]:
        """List of supported languages."""
        return self.__supported_languages

    def is_language_supported(self, language_code: LanguageCode) -> bool:
        """Is the language_code code supported by the engine.

        Parameters
        -----------
        language_code: str
            ISO-3166 Country Codes and ISO-639 Language Codes in the format '<language_code>_<country>, e.g., en_US.

        Returns
        -------
        flag: bool
            Flag if this language_code code is supported.
        """
        return language_code in self.supported_language

    def __repr__(self):
        return f'Personal Entity Linking:= {self.service_url}'


class NamedEntityRecognitionProcessor(WacomServiceAPIClient):
    """
    NamedEntityRecognitionProcessor
    -------------------------------
    Service that recognizes entities.

    Parameters
    ----------
    service_url: str
        URL where the service has been deployed
    supported_languages: List[str] = None
        List of supported languages
    verify_calls: bool (default:=False)
        Verifies all HTTPS calls and the associated certificate.
    """

    def __init__(self, service_url: str, supported_languages: List[LanguageCode] = None,
                 verify_calls: bool = False):
        super().__init__(application_name='Named Entity Linking', service_url=service_url, service_endpoint="graphdata",
                         verify_calls=verify_calls)
        self.__supported_languages: List[LanguageCode] = supported_languages if supported_languages else []

    @abc.abstractmethod
    def named_entities(self, text: str, language_code: LanguageCode = 'en_US') -> List[NamedEntity]:
        """
        Performs Named Entity Recognition on a text.

        Parameters
        ----------
        text: str
            Text where the entities shall be tagged in.
        language_code: LanguageCode
            ISO-3166 Country Codes and ISO-639 Language Codes in the format '<language_code>_<country>, e.g., en_US.

        Returns
        -------
        entities: List[NamedEntity]
            List of knowledge named entities.
        """
        raise NotImplemented

    @property
    def supported_language(self) -> List[LanguageCode]:
        """List of supported languages."""
        return self.__supported_languages

    def is_language_supported(self, language_code: LanguageCode) -> bool:
        """Is the language_code code supported by the engine.

        Parameters
        ----------
        language_code: LanguageCode
            ISO-3166 Country Codes and ISO-639 Language Codes in the format '<language_code>_<country>, e.g., en_US.

        Returns
        -------
        flag: bool
            Flag if this language_code code is supported
        """
        return language_code in self.supported_language

    def __repr__(self):
        return f'Public entity linking:= {self.__service_url}'


class PublicEntityLinkingProcessor(RESTAPIClient):
    """
    Public Entity Linking
    ---------------------
    Service that links entities to a public entities in a knowledge graph.

    Parameters
    ----------
    service_url: str
        URL where the service has been deployed
    supported_languages: List[str] = None
        List of supported languages
    verify_calls: bool (default:=False)
        Verifies all HTTPS calls and the associated certificate.
    """

    def __init__(self, service_url: str = str, supported_languages: List[str] = None, verify_calls: bool = False):
        super().__init__(service_url=service_url, verify_calls=verify_calls)
        self.__supported_languages: List[str] = supported_languages if supported_languages else []

    @abc.abstractmethod
    def link_public_entities(self, text: str, language_code: LanguageCode = 'en_US') -> List[KnowledgeGraphEntity]:
        """
        Performs Named Entity Linking on a text. It only finds entities within a large public knowledge graph.

        Parameters
        ----------
        text: str
            Text where the entities shall be tagged in.
        language_code: LanguageCode
            ISO-3166 Country Codes and ISO-639 Language Codes in the format '<language_code>_<country>, e.g., en_US.

        Returns
        -------
        entities: List[KnowledgeGraphEntity]
            List of knowledge public knowledge entities.
        """
        raise NotImplemented

    @property
    def supported_language(self) -> List[str]:
        """List of supported languages."""
        return self.__supported_languages

    def is_language_supported(self, language_code: str) -> bool:
        """
        Is the language_code code supported by the engine.

        Parameters
        ----------
        language_code: LanguageCode
            ISO-3166 Country Codes and ISO-639 Language Codes in the format '<language_code>_<country>, e.g., en_US.

        Returns
        -------
        flag: bool
            Flag if this language_code code is supported
        """
        return language_code in self.supported_language

    def __repr__(self):
        return f'Public Entity Linking:= {self.service_url}'



#!/usr/bin/env python
import pathlib

from setuptools import setup, find_packages

import sys
CURRENT_PYTHON = sys.version_info[:2]
REQUIRED_PYTHON = (3, 6)

if CURRENT_PYTHON < REQUIRED_PYTHON:
    sys.stderr.write("""
==========================
Unsupported Python version
==========================
This version of personal knowledge lib requires Python {}.{}, but you're trying to
install it on Python {}.{}.
This may be because you are using a version of pip that doesn't
understand the python_requires classifier. Make sure you
have pip >= 9.0 and setuptools >= 24.2, then try again:
    $ python -m pip install --upgrade pip setuptools
This will install the latest version of knowledge-service-lib which works on your
version of Python. If you can't upgrade your pip (or Python), request
an older version of knowledge-service-lib :
    $ python -m pip install wacom-knowledge-services-library
""".format(*(REQUIRED_PYTHON + CURRENT_PYTHON)))
    sys.exit(1)

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

# the setup
setup(
    name='personal_knowledge_library',
    version='0.1.5',
    description="Library to access Wacom's Personal Knowledge graph.",
    long_description=README,
    long_description_content_type="text/markdown",
    url='https://github.com/Wacom-Developer/personal-knowledge-library',
    author='Markus Weber',
    author_email='markus.weber@wacom.com',
    license='Apache 2.0 License',
    keywords='semantic-knowledge;knowledge-graph',
    packages=find_packages(exclude=('docs', 'tests', 'env')),
    include_package_data=True,
    install_requires=[
        "requests>=2.25.1",
        "qwikidata>=0.4.0",
        "python-dateutil>=2.8.2",
        "tqdm>=4.62.0",
        "ndjson>=0.3.1",
        "rdflib==6.0.2"
    ],
    extras_require={
    },
    tests_require=(
        'pytest',
        'pytest-cov'
    ),
    classifiers=[],
    )



# -*- coding: utf-8 -*-
# Copyright © 2021 Wacom. All rights reserved.
from typing import Dict, List, Any, Tuple, Union
import enum

import requests
from requests import Response

from knowledge.services.base import WacomServiceAPIClient, WacomServiceException


# -------------------------------------- Constant flags ----------------------------------------------------------------
TENANT_ID: str = 'tenantId'
USER_ID_TAG: str = 'userId'
LIMIT_TAG: str = 'limit'
OFFSET_TAG: str = 'offset'
ROLES_TAG: str = 'roles'
META_DATA_TAG: str = 'metaData'
INTERNAL_USER_ID_TAG: str = 'internalUserId'
EXTERNAL_USER_ID_TAG: str = 'externalUserId'
CONTENT_TYPE_FLAG: str = 'Content-Type'
TENANT_API_KEY_FLAG: str = 'x-tenant-api-key'


class UserRole(enum.Enum):
    """
    UserRole
    --------
    Roles of the users in
    """
    USER = 'User'
    """User only has control over his personal entities."""
    ADMIN = 'TenantAdmin'
    """TenantAdmin has access to all entities independent of the access rights."""


USER_ROLE_MAPPING: Dict[str, UserRole] = dict([(r.value, r) for r in UserRole])


class UserManagementServiceAPI(WacomServiceAPIClient):
    """
    User-Management Service API
    -----------------------------

    Functionality:
        - List all users
        - Create / update / delete users

    Parameters
    ----------
    service_url: str
        URL of the service
    service_endpoint: str
        Base endpoint
    """

    USER_DETAILS_ENDPOINT: str = f'{WacomServiceAPIClient.USER_ENDPOINT}/internal-id'
    SERVICE_URL: str = 'https://semantic-ink-private.wacom.com'

    def __init__(self, service_url: str = SERVICE_URL, service_endpoint: str = 'graphdata'):
        super().__init__("GroupManagementServiceAPI", service_url=service_url, service_endpoint=service_endpoint)

    # ------------------------------------------ Users handling --------------------------------------------------------

    def create_user(self, tenant_key: str, external_id: str, meta_data: Dict[str, str] = None,
                    roles: List[UserRole] = None) -> Tuple[str, str, str, Dict[str, Any], List[UserRole]]:
        """
        Creates user for a tenant.

        Parameters
        ----------
        tenant_key: str -
            API key for tenant
        external_id: str -
            External id of user identification service.
        meta_data: Dict[str, str]
            Meta-data dictionary.
        roles: List[UserRole]
            List of roles.

        Returns
        -------
        user_id: str
            User  Id
        tenant_id: str
            Tenant Id
        external_user_id: str
            External user Id
        meta_data: Dict[str, Any]
            Meta data for user.
        roles: List[UserRole]
            List of roles.

        Raises
        ------
        WacomServiceException
            If the tenant service returns an error code.
        """
        url: str = f'{self.service_base_url}{UserManagementServiceAPI.USER_ENDPOINT}'
        headers: dict = {
            TENANT_API_KEY_FLAG: tenant_key,
            CONTENT_TYPE_FLAG: 'application/json'
        }
        payload: dict = {
            EXTERNAL_USER_ID_TAG: external_id,
            META_DATA_TAG: meta_data if meta_data is not None else {},
            ROLES_TAG: [r.value for r in roles] if roles is not None else [UserRole.USER.value]
        }
        response: Response = requests.post(url, headers=headers, json=payload, verify=self.verify_calls)
        if response.ok:
            results: Dict[str, Union[str, Dict[str, str], List[str]]] = response.json()
            user_id: str = results['id']
            tenant_id: str = results['tenantId']
            external_user_id: str = results['externalUserId']
            meta_data: Dict[str, Any] = results['metaData']
            user_roles: List[UserRole] = [USER_ROLE_MAPPING[r] for r in results['roles']]
            return user_id, tenant_id, external_user_id, meta_data, user_roles

        raise WacomServiceException(f'Response code:={response.status_code}, exception:= {response.text}')

    def update_user(self, tenant_key: str, internal_id: str, external_id: str, meta_data: Dict[str, str] = None,
                    roles: List[UserRole] = None):
        """Updates user for a tenant.

        Parameters
        ----------
        tenant_key: str
            API key for tenant
        internal_id: str
            Internal id of semantic service.
        external_id: str
            External id of user identification service.
        meta_data: Dict[str, str]
            Meta-data dictionary.
        roles: List[UserRole]
            List of roles.

        Raises
        ------
        WacomServiceException
            If the tenant service returns an error code.
        """
        url: str = f'{self.service_base_url}{UserManagementServiceAPI.USER_ENDPOINT}'
        headers: Dict[str, str] = {
            TENANT_API_KEY_FLAG: tenant_key,
            CONTENT_TYPE_FLAG: 'application/json'
        }
        payload: Dict[str, str] = {
            META_DATA_TAG: meta_data if meta_data is not None else {},
            ROLES_TAG: [r.value for r in roles] if roles is not None else [UserRole.USER.value]
        }
        params: Dict[str, str] = {
            USER_ID_TAG: internal_id,
            EXTERNAL_USER_ID_TAG: external_id
        }
        response: Response = requests.patch(url, headers=headers, json=payload, params=params, verify=self.verify_calls)
        if not response.ok:
            raise WacomServiceException(f'Updating user failed. '
                                        f'Response code:={response.status_code}, exception:= {response.text}')

    def delete_user(self, tenant_key: str, external_id: str, internal_id: str):
        """Deletes user from tenant.

        Parameters
        ----------
        tenant_key: str
            API key for tenant
        external_id: str
            External id of user identification service.
        internal_id: str
            Internal id of user.

        Raises
        ------
        WacomServiceException
            If the tenant service returns an error code.
        """
        url: str = f'{self.service_base_url}{UserManagementServiceAPI.USER_ENDPOINT}'
        headers: Dict[str, str] = {
            TENANT_API_KEY_FLAG: tenant_key
        }
        params: Dict[str, str] = {
            USER_ID_TAG: internal_id,
            EXTERNAL_USER_ID_TAG: external_id
        }
        response: Response = requests.delete(url, headers=headers, params=params, verify=self.verify_calls)
        if not response.ok:
            raise WacomServiceException(f'Response code:={response.status_code}, exception:= {response.text}')

    def user_internal_id(self, tenant_key: str, external_id: str) -> Dict[str, Any]:
        """User internal id.

        Parameters
        ----------
        tenant_key: str
            API key for tenant
        external_id: str
            External id of user

        Returns
        -------
        internal_user_id: str
            Internal id of users

        Raises
        ------
        WacomServiceException
            If the tenant service returns an error code.
        """
        url: str = f'{self.service_base_url}{UserManagementServiceAPI.USER_DETAILS_ENDPOINT}'
        headers: dict = {
            TENANT_API_KEY_FLAG: tenant_key
        }
        parameters: Dict[str, str] = {
            EXTERNAL_USER_ID_TAG:  external_id
        }
        response: Response = requests.get(url, headers=headers, params=parameters, verify=self.verify_calls)
        if response.ok:
            response_dict: Dict[str, Any] = response.json()
            return response_dict[INTERNAL_USER_ID_TAG]
        raise WacomServiceException(f'Response code:={response.status_code}, exception:= {response.text}')

    def listing_users(self, tenant_key: str, offset: int = 0, limit: int = 20)\
            -> List[Tuple[str, str, str, Dict[str, Any], List[UserRole]]]:
        """
        Listing all users configured for this instance.

        Parameters
        ----------
        tenant_key: str
            API key for tenant
        offset: int - [optional]
            Offset value to define starting position in list. [DEFAULT:= 0]
        limit: int - [optional]
            Define the limit of the list size. [DEFAULT:= 20]

        Returns
        -------
        user:  List[Tuple[str, str, str, Dict[str, Any], List[UserRole]]]
            List of users.
        """
        url: str = f'{self.service_url}/{self.service_endpoint}{UserManagementServiceAPI.USER_ENDPOINT}'
        headers: Dict[str, str] = {
            TENANT_API_KEY_FLAG: tenant_key
        }
        params: Dict[str, str] = {
            OFFSET_TAG: offset,
            LIMIT_TAG: limit
        }
        response: Response = requests.get(url, headers=headers, params=params, verify=self.verify_calls)
        if response.ok:
            users: List[Dict[str, Any]] = response.json()
            results: List[Tuple[str, str, str, Dict[str, Any], List[UserRole]]] = []
            for u in users:
                user_id: str = u['id']
                tenant_id: str = u[TENANT_ID]
                external_user_id: str = u['externalUserId']
                meta_data: Dict[str, Any] = u['metaData']
                user_roles: List[UserRole] = [USER_ROLE_MAPPING[r] for r in u['roles']]
                results.append((user_id, tenant_id, external_user_id, meta_data, user_roles))
            return results
        raise WacomServiceException(f'Listing of users failed.'
                                    f'Response code:={response.status_code}, exception:= {response.text}')
